YOMP_CQRS
=========

This library provides Command/Query Responsibility Segregation engine, which is connected to YOMP_DB for storing data. 

See `test` folder for a lot of examples of usage.

Event sourcing
--------------
The key element for highly distributed and collaborative environments. Basic idea is that objects are stored as
series of events. Event for create, set property, add to array, remove from array etc.. Upon replaying those events
the system builds the particular object. Any change is just creating small event and send to all listeners. They, by 
replaying, will update their models. The system will never change the event in the past.

Building of objects from event stream is responsibility of Projections. It's clear that there could be several
projections for one stream, resulting in different representations of one stream. 

```dart
import "package:yomp_db/storages/memory.dart" as ydb_mems;
import "package:yomp_cqrs/yomp_cqrs.dart" as ycq;
...
ycq.EventSourcing es = new ycq.EventSourcing(new ydb_mems.StorageMemory());
await es.open("test");
List events = [
{"event":"Genre.create","id":"METAL"},
{"event":"Genre.set_name","id":"METAL", "value":"Heavy metal"},
{"event":"Band.create","id":"IRONS"},
{"event":"Band.set_name","id":"IRONS", "value":"Iron Maiden"},
{"event":"Band.set_genre","id":"IRONS", "value":"METAL"},
];
await es.addEvents(events);
//EventSourcing object now contains 5 events
Map docs = await es.project(new ycq.ProjectionBasic());
// the docs contains two documents:
 {"_id":"METAL","type":"Genre","name":"Heavy Metal"}
 {"_id":"IRONS","type":"Band","name":"Iron Maiden","genre":"METAL"}
es.addEvents([{"event":"Band.set_name","id":"IRONS","value":"Iron 'Classics' Maiden"}]);
//EventSourcing object now contains 6 events
Map docs2 = await es.project(new ycq.ProjectionBasic());
// the docs2 contains two documents:
 {"_id":"METAL","type":"Genre","name":"Heavy Metal"}
 {"_id":"IRONS","type":"Band","name":"Iron 'Classics' Maiden","genre":"METAL"}
```

GIT
---
<https://bitbucket.org/tomas-rokos/yomp>

YOMP
---
Yomp is family of libraries created as framework for client/server development in Dart. The creation is based 
on pull model. The additions are done when needed, so there could be missing obvious methods, but they were 
just not needed so far.

*[Yomp](http://en.wikipedia.org/wiki/Yomp) is word for long-distance march carrying full kit.*