@TestOn("vm")

import "package:yomp_db/storages/memory.dart" as ydb_mems;
import "package:yomp_db/object_dumper.dart";
import "package:yomp_cqrs/yomp_cqrs.dart" as ycq;
import "package:test/test.dart";
import "tester_data_bands.dart";

void main() {
	group("Projection Basic -",() {
		ydb_mems.StorageMemory store;
		ycq.EventSourcing es;
		setUp(() async {
			store = new ydb_mems.StorageMemory();
			es = new ycq.EventSourcing(store);
			await es.open("test");
			List data = tester_data_bands();
			await es.addEvents(data);
		});
		tearDown(() async {
			await store.delete();
		});
		test("just projection", () async {
			Map result = await es.project(new ycq.ProjectionBasic());
			expect(result.length,30);
			List<String> ids = ["ROCK","METAL","GNR","IRONS","F1","F2","F3","F4","F5","F6","A1","A2",
								"BRUCED","STEVEH","DAVEM","ADRIANS","NICKOB","JANICKG",
								"PAULD","DENNISS","CLIVEB","BLAZEB",
								"AXELR","SLASH","IZZYS","DUFFM","STEVENA","DIZZYR","MATTS","GILBYC"
								];
			expect(result.keys,unorderedEquals(ids));
		});
		test("one property change",() async {
			Map result = await es.project(new ycq.ProjectionBasic());
			expect(result.length,30);
			expect(result["BRUCED"]["name"],"Bruce Dickinson");
			await es.addEvents([{"event":"Person.set_name","id":"BRUCED", "value":"Bruce 'Sirene' Dickinson"}]);
			result = await es.project(new ycq.ProjectionBasic());
			expect(result.length,30);
			expect(result["BRUCED"]["name"],"Bruce 'Sirene' Dickinson");
		});
		test("one property add",() async {
			Map result = await es.project(new ycq.ProjectionBasic());
			expect(result.length,30);
			expect(result["F5"]["guitar"],unorderedEquals(["SLASH","IZZYS"]));
			await es.addEvents([{"event":"Formation.add_guitar","id":"F5", "value":"BRUCED"}]);
			result = await es.project(new ycq.ProjectionBasic());
			expect(result.length,30);
			expect(result["F5"]["guitar"],unorderedEquals(["BRUCED","SLASH","IZZYS"]));
		});
	});
}